//                              -*- Mode: C++ -*-
//: EALinkAnalyzer.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 13/8/97
/* $Log: EALinkAnalyzer.C,v $
 * Revision 1.9  2000/01/11 11:24:36  natasha
 * care (3.50):
 * The major changes with respect to the previous release are:
 * 1) Digi MC inizialization is skipped for real data.
 * 2) the care/dglev command has been removed in favour of the hbdigi/dglev
 * 3) MC calibration constants retuned on MC data with magnet ON
 * 4) Cluster z correction retuned
 * 5) More flexibility for cluster threshold settings (ECCutFunction.C)
 * 6) The most important warning messages are counted
 * 7) No more chashes on missing DB
 * 8) Full C++ 9X standard conformance of the code
 *
 * Revision 1.8  1999/08/04 15:21:31  natasha
 * new care version from Mauro
 *
 * Revision 1.3  1999/06/01  16:07:33  villa
 * before HH
 *
 */

#include "care/care.hh"
#include "care/EMCluster.hh"
#include "care/EALinkAnalyzer.hh"
#include "arte/ArteMessage.hh"
#include "arte/ArteProto.hh"
#include "care/EMTrackOnEcal.hh"
#include "care/MMBank.hh"

#ifdef MCARLO
#  include "care/EAMCTruthAnalyzer.hh"
#endif
#undef WRITE
#undef DEBUG

/* 
CLASS
  EALinkAnalyzer
  Class for defining and updating the links between the
  RCCL bank and the banks HITC and RTRA.

*/

//////////////////// EALinkAnalyzer Class Definitions ////////////////

// No functions
EALinkAnalyzer::EALinkAnalyzer() : mkHitRel(0), mkTraRel(0), nrccl(0) {
  // initialize data for the relational bank
}

EALinkAnalyzer::~EALinkAnalyzer(){}


//--
// Cluster Analyzer
void EALinkAnalyzer::doAnalysis(const EMCluster * cluster){
  // loop on all cluster members in REVERSE directions since in this way the
  // relations are stored in DIRECT order...
  for(const EMClusterMember* child = cluster->rbegin(); 
      child!=cluster->rend();  child = child->previous())
    child->analyzeMe(*this);
}

//--
// ClusterTree Analyzer
void EALinkAnalyzer::doAnalysis(const EMClusterTree * ){}


//--
// HitCell Analyzer
void EALinkAnalyzer::doAnalysis(const EMHitCell *hitC){
# ifndef ONLINE
  if(mkHitRel){
    unsigned int i = hitC->whichBank().index();
    // write the relation for this cell
    if( i> ArtePointer<HITC>::nrow() || i==0 || 
	nrccl< 1|| nrccl>ArtePointer<RCCL>::nrow() ){
      arteMsg<<" Error: hitc: "<<i<<"  RCCL: "<<nrccl;
      arteMsg.warn();
      arteMsg<<"        hitc: "<< ArtePointer<HITC>::nrow()
	  <<"  RCCL: "<<ArtePointer<RCCL>::nrow();
      arteMsg.warn();
      hitC->getParent()->dump();
    }
    createRelation( hitC->whichBank(), ArtePointer<RCCL>(nrccl), &RCCL::hitc);
    //    MMBank<RCCL_HITC>::Brel(i, nrccl);
  }
# endif
}
//--
//  TrackOnEcal Analyzer
void EALinkAnalyzer::doAnalysis(const EMTrackOnEcal * track){
# ifndef ONLINE
  if(mkTraRel){
    // get the track index number
#   ifndef CMATCH
    ArtePointer<RSEG> rtx = track->Infos().track;
    unsigned int i = rtx.index();
#   else
    unsigned int i = track->Infos()->link().index();
#   endif

#   ifdef RTRALINK
    ArtePointer<RTRA> rtx = track->Infos()->link();
    if( i> ArtePointer<RTRA>::nrow() || i==0 || 
	nrccl< 1|| nrccl>ArtePointer<RCCL>::nrow()){
      arteMsg<<" Error: track: "<<i<<"  RCCL: "<<nrccl;
      arteMsg.warn();
      arteMsg<<"        rtra: "<< ArtePointer<RTRA>::nrow()
	  <<"  RCCL: "<<ArtePointer<RCCL>::nrow();
      arteMsg.warn();
      track->getParent()->dump();
    }
    // write the relation for this cell
    //    MMBank<RCCL_RTRA>::Brel(i, nrccl);
    createRelation( rtx, ArtePointer<RCCL>(nrccl), &RCCL::rtra);
#  else
    ArtePointer<RSEG> rtx = track->Infos()->link();
    if( i> ArtePointer<RSEG>::nrow() || i==0 || 
	nrccl< 1|| nrccl>ArtePointer<RCCL>::nrow()){
      arteMsg<<" Error: track: "<<i<<"  RCCL: "<<nrccl;
      arteMsg.warn();
      arteMsg<<"        rseg: "<< ArtePointer<RSEG>::nrow()
	     <<"  RCCL: "<<ArtePointer<RCCL>::nrow();
      arteMsg.warn();
      track->getParent()->dump();
    }
    // write the relation for this cell
    //    MMBank<RCCL_RSEG>::Brel(i, nrccl);
    createRelation( rtx, ArtePointer<RCCL>(nrccl), &RCCL::rseg );
#  endif
  }
# endif
}

//--
// EMClusterMember Analyzer
void EALinkAnalyzer::doAnalysis(const EMClusterMember *){}

//--
//  Fill totally new relations
//
void EALinkAnalyzer::fillNewEntry(const EMCluster * cluster, 
				  ArtePointer<RCCL> rccl){
  // now build relational bank RCCL_HITC and RCCL_RTRA
  mkHitRel = true;
  mkTraRel = true;
  nrccl = rccl.index();
  doAnalysis(cluster);
  mkHitRel = false;
  mkTraRel = false;

# ifdef WRITE
  testLinks(cluster, rccl);
# endif

# ifdef MCARLO
  EAMCTruthAnalyzer mct;
  mct.reset();
  mct.doAnalysis(cluster);
  mct.sortDeposits();
  ECMCDeposit  dep = mct.maxDeposit();
  // Evaluate the total deposit in a cell
  Energy_t edep = 0.0;
  std::vector<ECMCDeposit>::const_iterator first = mct.getVector().begin();
  std::vector<ECMCDeposit>::const_iterator last = mct.getVector().end();
  for(std::vector<ECMCDeposit>::const_iterator iter = first; iter!=last;
	iter++){
    edep += iter->energy();
  }
  // Evaluate the 80 % of the deposit
  Energy_t edep1 = edep* 0.80;
  std::vector<ECMCDeposit>::const_iterator iter1 = first;
  for(; iter1!=last && edep1>0.0; iter1++){
    edep1 -= iter1->energy();
  }
  last=iter1;

  std::vector<ECMCDeposit>::const_reverse_iterator rvfirst = 
	 mct.getVector().rend();
  for( std::vector<ECMCDeposit>::const_reverse_iterator rviter1 = 
	 std::vector<ECMCDeposit>::const_reverse_iterator(last);
       rviter1!=rvfirst; rviter1++){
    
    // get the track index number
    unsigned int i = (*rviter1).track().index();
    if( i> ArtePointer<MTRA>::nrow() || 
	nrccl<1 || nrccl>ArtePointer<RCCL>::nrow()){
      arteMsg<<" Error: MC track: "<<i<<"  RCCL: "<<nrccl;
      arteMsg.warn();
      arteMsg<<"        mtra: "<< ArtePointer<MTRA>::nrow()
	     <<"  RCCL: "<<ArtePointer<RCCL>::nrow();
      arteMsg.warn();
      cluster->dump();
    } else if( i!=0 ) {
      createRelation( (*rviter1).track(), 
		      ArtePointer<RCCL>(nrccl), &RCCL::mtra);
      //      MMBank<RCCL_MTRA>::Brel(i, nrccl);
    }
  }
# endif
}

//--
//  Evaluate the information for the cluster 
//  and fill the TRACK relation bank only
//

void EALinkAnalyzer::updateTrackRelEntry(const EMCluster * cluster, 
					 ArtePointer<RCCL> rccl){
  // now build relational bank RCCL_RTRA
  mkHitRel = false;
  mkTraRel = true;
  nrccl = rccl.index();
  doAnalysis(cluster);
  mkHitRel = false;
  mkTraRel = false;

# ifdef WRITE
  testLinks(cluster, rccl);
# endif
}

//--
//  Evaluate the information for the cluster and 
//  fill an old entry in the bank
//
void EALinkAnalyzer::updateOldEntry(const EMCluster * , 
				    ArtePointer<RCCL> ){}

//--
//  Test the links
//
#ifndef DEBUG
void EALinkAnalyzer::testLinks(const EMCluster * , ArtePointer<RCCL> ){
 //
#else
void EALinkAnalyzer::testLinks(const EMCluster * clus, 
			       ArtePointer<RCCL> rccl){
 //
  arteMsg<<"Testing cluster";
  arteMsg.allp();
  clus->dump();
  // test the link done
  arteMsg<<"Cluster number:"<<rccl.index()
      <<" at "<<rccl->x<<","<<rccl->y
      <<" with "<<rccl->wid<<" cells and "<<rccl->esum<<" GeV";
  arteMsg.allp();
  // now loop on relations of this RCCL with HITC
  for( ArteRelationIterator<HITC> relIter((rccl->hitc).begin());
       relIter!=(rccl->hitc).end(); ++relIter){
    ArtePointer<HITC> cell = ArtePointer<HITC>(relIter);
    arteMsg<<"->Cell: ID="<< cell->gcce<<"  Energy = "<<cell->q<<"  Pos:("
	<<cell->x<<", "<<cell->y<<") ";
    arteMsg.allp();
  }
  // now loop on relations of this RCCL with RTRA
  for( ArteRelationIterator<RTRA> relIterat((rccl->rtra).begin());
       relIterat!=(rccl->rtra).end(); ++relIterat){
    ArtePointer<RTRA> rtra = ArtePointer<RTRA>(relIterat);
    arteMsg<<"->Track: ID="<<rtra<<"  Momentum = "<< 1./rtra->pf;
    arteMsg.allp();
  }
#endif
}

