//                              -*- Mode: C++ -*-
//: EMClusterMember.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 3/4/97
// Last Update ........
//

#ifndef EMCLUSTERMEMBER_HH_
#define EMCLUSTERMEMBER_HH_

#include "care/ECALtypes.hh"

class EAClusterAnalyzer;  // tell compiler that a  EAClusterAnalyzer class exists
class EDClusterModifier;  // tell compiler that a  EDClusterModifier class exists 
class EMCluster;


/**
class  EMClusterMember.
  Pure abstract base class for the hierarchical cluster tree used in
  pattern recognition of ECAL. The tree is made with an underling structure
  of double linked lists. The EMClusterMember objects are actually the
  NODE and the DATA in these lists. For this reason each EMClusterMember
  has links to the previous and the next elements in the lists as well as
  the link to the list (the EMCluster object) having the object as its element.

Tree building:
  Left to the class ECBuilder.

Tree Iteration: 
  Analysis and traversing of the cluster tree is done by EAClusterAnalyzers
  that are object with definite analyzing properties. 
  EMClusterMembers MUST ignore these properties: in this way a loose coupling
  between EMClusterMembers and EAClusterAnalyzers is achieved and both
  can vary independently during program evolution.
  Examples of EAClusterAnalyzers are those that should return
  total energy of a cluster, the Center of Gravity, the Number of Cells.
  
*/

class EMClusterMember{
public:
  /**@name Constructors and Destructors */
  //@{
  /** Constructors.
      The construction of EMClusterMember is done referring to some
      parent. The depth level is adjusted accordingly. */
  EMClusterMember( EMCluster* parent);
  //  EMClusterMember();
  /// Destructor:  A virtual destructor is needed.
  virtual ~EMClusterMember();
  //@}

  /**@name Status access functions */
  //@{
  /** Return a flag to tell if this EMClusterMember is a leaf 
    (if it is not, then it can be a clustermember list). */
  virtual bool isALeaf() const = 0;

  /// Get Parent pointer.
  EMCluster*  getParent() const { return parentCluster;}

  //--
  /** Return the maximum energy measured in the cluster's (and sub clusters) 
      cells or the Energy of the Cell itself. */
  virtual Energy_t  maxEnergy() const = 0;
  //--
  /// return depth level.
  Level_t depthLevel() const { return depth;}
  //--
  ///  Return next child.
  EMClusterMember * next() const { return nextClusterMember;}
  //--
  ///  Return previous child.
  EMClusterMember * previous() const { return previousClusterMember;}

  //--
  ///  Dump for debugging.
  virtual void dump() const;
  //@}


  /**@name Beheavioral methods */
  //@{
  //--
  /// Add a Child to the list: it makes sense only for cluster objects.
  virtual  EMClusterMember* add(EMClusterMember* child);  
  //--
  /** Append the current child (unititialized for links)  
    to the list defined by the parent cluster. */
  EMClusterMember* appendTo(EMCluster* parent);  
  
  //--
  /** After operation on cluster such as reordering, merging or other
    It is possible that the depths are not correct. With this function
    the depth of the EMClusterMember and all its children are adjusted. */ 
  virtual void adjust_depths();

  /** 
    Analysis and traversing of the cluster tree is done by EAClusterAnalyzers
   that are object with definite analyzing properties.
   EMClusterMembers MUST ignore these properties: in this way a loose coupling
   between EMClusterMembers and EAClusterAnalyzers is achieved and both
   can vary independently during program evolution.
   Examples of EAClusterAnalyzers are those that should return
   total energy of a cluster, the Center of Gravity, the Number of Cells. */
  virtual void modifyMe(EDClusterModifier & analyzer);
  
  /**
   Analysis and traversing of the cluster tree is done by EAClusterAnalyzers
   that are object with definite analyzing properties. 
   EMClusterMembers MUST ignore these properties: in this way a loose coupling
   between EMClusterMembers and EAClusterAnalyzers is achieved and both
   can vary independently during program evolution.
   Examples of EAClusterAnalyzers are those that should return
   total energy of a cluster, the Center of Gravity, the Number of Cells. */ 
  virtual void analyzeMe(EAClusterAnalyzer & analyzer) const;
  
 
  /**
     Follow upstream all the links until level "lev" is reached
     return 0 or the EMCluster pointer in which level "lev" is. */
  EMCluster* upstream(Level_t level) const;
  //@}
  friend class EMCluster;

protected:
  /**
    The default constructors are protected in order to prevent
    wrong usage of this class by user. */
  EMClusterMember();
  /**@name Private Data */
  //@{
  /** 
     This is a link to the parent of the current node.
     If there is no parent its value is NULL. */
  EMCluster* parentCluster;
  
  
  ///  Use for the linked list
  EMClusterMember* nextClusterMember;
  EMClusterMember* previousClusterMember;

  ///  Deepness level for fast evaluation in analysis.
  Level_t  depth;
  //@}

private:

};


typedef EMClusterMember ClusterMember;

#endif // CLUSTERMEMBER_HH_




