/* Last versions of STL provide this functionality 
   for all needed const/non const versions. Please use standard version
   instead of this hand made one.
-----------------------------------------------------------------------------

   Use these staff in one of two forms:

         member_function(      Object& object, F f, (Arg1*)0, ..., (ArgN*)0); 
   const_member_function(const Object& object, F f, (Arg1*)0, ..., (ArgN*)0);

   where (object.*f) - some member function which you would like to call
   for member "object". It should take 0<=N<=5 arguments and
   expression (object.*f)(a1, .., aN) 
   (a1, .., aN are of types Arg1, .., ArgN respectively) 
   should be valid.
   member_function(..) returns an object of type MemberFunction0..5<..>,
   const_member_function(..) returns ConstMemberFunction0..5<..>. Both of them
   have the operator()(A1 a1, .., AN aN) which just calls 
   (object.*f)(a1, .., aN)
*/

// Author V.Balagura (balagura@mail.desy.de)   (09.04.98).

#ifndef _member_function_hh
#define _member_function_hh 1

/* -------------------- 0 args -------------------- */
template <class Object, class F>
class MemberFunction0 {
protected:
  Object& object;
  F func;
public:
  MemberFunction0(Object& obj, F f) : object(obj), func(f) {}
  void operator()() { (object.*func)(); }
};
template <class Object, class F>
inline MemberFunction0<Object, F> member_function(Object& obj, F f) {
  return MemberFunction0<Object, F>(obj, f);
}
/* ..... const_member_function ..... */
template <class Object, class F>
class ConstMemberFunction0 {
protected:
  const Object& object;
  F func;
public:
  ConstMemberFunction0(const Object& obj, F f) : object(obj), func(f) {}
  void operator()() { (object.*func)(); }
};
template <class Object, class F>
inline ConstMemberFunction0<Object, F> 
const_member_function(const Object& obj, F f) {
  return ConstMemberFunction0<Object, F>(obj, f);
}
/* -------------------- 1 args -------------------- */
template <class Object, class F, class A1>
class MemberFunction1 {
protected:
  Object& object;
  F func;
public:
  MemberFunction1(Object& obj, F f) : object(obj), func(f) {}
  void operator()(A1 a1) { (object.*func)(a1); }
};
template <class Object, class F, class A1>
inline MemberFunction1<Object, F, A1> member_function(Object& obj, F f, A1*) {
  return MemberFunction1<Object, F, A1>(obj, f);
}
/* ..... const_member_function ..... */
template <class Object, class F, class A1>
class ConstMemberFunction1 {
protected:
  const Object& object;
  F func;
public:
  ConstMemberFunction1(const Object& obj, F f) : object(obj), func(f) {}
  void operator()(A1 a1) { (object.*func)(a1); }
};
template <class Object, class F, class A1>
inline ConstMemberFunction1<Object, F, A1> 
const_member_function(const Object& obj, F f, A1*) {
  return ConstMemberFunction1<Object, F, A1>(obj, f);
}
/* -------------------- 2 args -------------------- */
template <class Object, class F, class A1, class A2>
class MemberFunction2 {
protected:
  Object& object;
  F func;
public:
  MemberFunction2(Object& obj, F f) : object(obj), func(f) {}
  void operator()(A1 a1, A2 a2) { (object.*func)(a1, a2); }
};
template <class Object, class F, class A1, class A2>
inline MemberFunction2<Object, F, A1, A2> member_function
(Object& obj, F f, A1*, A2*) {
  return MemberFunction2<Object, F, A1, A2>(obj, f);
}
/* ..... const_member_function ..... */
template <class Object, class F, class A1, class A2>
class ConstMemberFunction2 {
protected:
  const Object& object;
  F func;
public:
  ConstMemberFunction2(const Object& obj, F f) : object(obj), func(f) {}
  void operator()(A1 a1, A2 a2) { (object.*func)(a1, a2); }
};
template <class Object, class F, class A1, class A2>
inline ConstMemberFunction2<Object, F, A1, A2> 
const_member_function(const Object& obj, F f, A1*, A2*) {
  return ConstMemberFunction2<Object, F, A1, A2>(obj, f);
}
/* -------------------- 3 args -------------------- */
template <class Object, class F, class A1, class A2, class A3>
class MemberFunction3 {
protected:
  Object& object;
  F func;
public:
  MemberFunction3(Object& obj, F f) : object(obj), func(f) {}
  void operator()(A1 a1, A2 a2, A3 a3) {
    (object.*func)(a1, a2, a3); 
  }
};
template <class Object, class F, class A1, class A2, class A3>
inline MemberFunction3<Object, F, A1, A2, A3> member_function
(Object& obj, F f, A1*, A2*, A3*) {
  return MemberFunction3<Object, F, A1, A2, A3>(obj, f);
}
/* ..... const_member_function ..... */
template <class Object, class F, class A1, class A2, class A3>
class ConstMemberFunction3 {
protected:
  const Object& object;
  F func;
public:
  ConstMemberFunction3(const Object& obj, F f) : object(obj), func(f) {}
  void operator()(A1 a1, A2 a2, A3 a3) { 
    (object.*func)(a1, a2, a3); 
  }
};
template <class Object, class F, class A1, class A2, class A3>
inline ConstMemberFunction3<Object, F, A1, A2, A3> 
const_member_function(const Object& obj, F f, A1*, A2*, A3*) {
  return ConstMemberFunction3<Object, F, A1, A2, A3>(obj, f);
}
/* -------------------- 4 args -------------------- */
template <class Object, class F, class A1, class A2, class A3, class A4>
class MemberFunction4 {
protected:
  Object& object;
  F func;
public:
  MemberFunction4(Object& obj, F f) : object(obj), func(f) {}
  void operator()(A1 a1, A2 a2, A3 a3, A4 a4) {
    (object.*func)(a1, a2, a3, a4); 
  }
};
template <class Object, class F, class A1, class A2, class A3, class A4>
inline MemberFunction4<Object, F, A1, A2, A3, A4> member_function
(Object& obj, F f, A1*, A2*, A3*, A4*) {
  return MemberFunction4<Object, F, A1, A2, A3, A4>(obj, f);
}
/* ..... const_member_function ..... */
template <class Object, class F, class A1, class A2, class A3, class A4>
class ConstMemberFunction4 {
protected:
  const Object& object;
  F func;
public:
  ConstMemberFunction4(const Object& obj, F f) : object(obj), func(f) {}
  void operator()(A1 a1, A2 a2, A3 a3, A4 a4) { 
    (object.*func)(a1, a2, a3, a4); 
  }
};
template <class Object, class F, class A1, class A2, class A3, class A4>
inline ConstMemberFunction4<Object, F, A1, A2, A3, A4> 
const_member_function(const Object& obj, F f, A1*, A2*, A3*, A4*) {
  return ConstMemberFunction4<Object, F, A1, A2, A3, A4>(obj, f);
}
/* -------------------- 5 args -------------------- */
template <class Object, class F, class A1, class A2, class A3, class A4,
  class A5>
class MemberFunction5 {
protected:
  Object& object;
  F func;
public:
  MemberFunction5(Object& obj, F f) : object(obj), func(f) {}
  void operator()(A1 a1, A2 a2, A3 a3, A4 a4, A5 a5) {
    (object.*func)(a1, a2, a3, a4, a5); 
  }
};
template <class Object, class F, class A1, class A2, class A3, class A4,
  class A5>
inline MemberFunction5<Object, F, A1, A2, A3, A4, A5> member_function
(Object& obj, F f, A1*, A2*, A3*, A4*, A5*) {
  return MemberFunction5<Object, F, A1, A2, A3, A4, A5>(obj, f);
}
/* ..... const_member_function ..... */
template <class Object, class F, class A1, class A2, class A3, class A4, 
  class A5>
class ConstMemberFunction5 {
protected:
  const Object& object;
  F func;
public:
  ConstMemberFunction5(const Object& obj, F f) : object(obj), func(f) {}
  void operator()(A1 a1, A2 a2, A3 a3, A4 a4, A5 a5) { 
    (object.*func)(a1, a2, a3, a4, a5); 
  }
};
template <class Object, class F, class A1, class A2, class A3, class A4, 
  class A5>
inline ConstMemberFunction5<Object, F, A1, A2, A3, A4, A5> 
const_member_function(const Object& obj, F f, A1*, A2*, A3*, A4*, A5*) {
  return ConstMemberFunction5<Object, F, A1, A2, A3, A4, A5>(obj, f);
}

#endif
