#ifndef __PHOTONDETECTORSUPERMODULE_H__
#define __PHOTONDETECTORSUPERMODULE_H__


// ------------------------------------------------------------------------

/// description of a supermodule
/**
  A supermodule is described by the class PhotonDetectorSuperModule.
  The front surface of the module is described as an rectangle. 
  The rectangular surface is subdivided into a certain number of rows 
  and columns (32 x 12) and has coordinate system with the x-direction 
  parallel to the short edge, the y-axis is parallel to the long edge 
  of the module.
  For the upper supermodules the y-axis points upwards and the x-axis
  is (nearly) parallel to the HERA-B x-axis. The origin is the lower 
  left corner. For the lower ones the y-axis points downwards and the
  x-axis is (nearly) antiparallel to the HERA-B x-axis. The origin is
  is the upper right corner. In this coordinate systems the index of the
  row and columns are always positiv (incl. zero) and the symmetrie between
  lower and upper half is respected. The momentum vector of the infalling
  photons has it largest component parallel to the z-axis of this coordinate
  system.
*/
class PhotonDetectorSuperModule : public Rectangle {
PhotonDetectorCell **pdcell;
int numcell;

protected:
  int iid;
  inline PhotonDetectorSuperModule() { pdcell=0;};

public:
/// detection of a photon, which hits this cell of the supermodule. See also page:  
/**
  The function "dodetect" is called to detect a photon. It search the hit
  cell, through a determination of the hit row and column. After that
  it ask the cell, if the photon is detected, by calling the "dodetect"
  member function of the class PhotonDetectorCell.
  The return value indicated, if the photon is detected (return value=1)
  or not (return value=0). In the first case the channelid is return
  with the last argument.
*/
  int dodetect(Photon* p,int *daqid);
/// returns the best-guess position of a hit, if supplied with the row,column of the cell and the pmt-channel
  Vec3 pos(int row,int col,int channel) const;
/// returns a pointer to the cell information and a {\bf zero} pointer, if the cell is empty (!).
  inline  PhotonDetectorCell* Cell(int row,int col) const {
    return pdcell[index(row,col)];
    }
/// returns a pointer to the cell information and a {\bf zero} pointer, if the cell is empty (!).
  inline  PhotonDetectorCell* Cell(double x,double y) const {
    return pdcell[index(x,y)];
    }

/// returns the identifier of the supermodule
  inline int id() const { return iid; }

/// returns supermodule row/column
  inline int smrow() const { return (id()>6) ? 0 : 1; }
  inline int smcol() const { return (id()>6) ? 13-id() : id(); }

///  print the contents of the class to an ASCII-File. 
  void print(FILE* fp=stdout);
  
/// constructor reading information from an ASCII-File produced using 'print'. See also page:
/** constructor reading information from an ASCII-File produced using 'print'
 To set the PM information a List containg all available PMT's 
 have to be supplied in the second argument.
  */   
  PhotonDetectorSuperModule(ScriptFile& f,
      List<PM>& PMList);
      
/// constructor using a record in the CSUP-ARTE-Table. See also page:
/** constructor using a record in the CSUP-ARTE-Table.
 To set the PM information a List containg all available PMT's 
 have to be supplied in the second argument.
  */   
  PhotonDetectorSuperModule( const ArtePointer<CSUP>& a,
      List<PM>& PMList);
/// destructor
  ~PhotonDetectorSuperModule();  
  };

#endif
